#include<stdio.h>
#include<limits.h>
#include<stdlib.h>
#include<time.h>

#include "base.h"
#include "field.h"
#include "mvalue.h"

static char row_e, col_e, diag_e;

static char countpentad (char *src, int len, char *dest)
{
  int i, num;

  num = 0;
  if(len < WINLEN) return 0;
  for(i=0; i<WINLEN; i++) if(src[i]) num++;
  dest[num]++;
  for(; i<len; i++){
    if(src[i-WINLEN]) num--;
    if(src[i]) num++;
    dest[num]++;
  }
  if(dest[WINLEN-1] || dest[WINLEN-2] > 1) return 3;
  if(dest[WINLEN-2] || dest[WINLEN-3] > 1) return 1;

  return 0;
}

static char countsquare (int x, int y, char player, char *dest)
{
  char row[2*WINLEN-1];
  int i, len, action;

  player = -player;
  for(i=0; i < WINLEN; i++) dest[i] = 0;

  action = 0;

  if(row_e){
    for(i = -1; i > -WINLEN && player != getsquare(x+i, y); i--);
    i++;
    for(len = 0; i+len < WINLEN && player != (row[len] = getsquare(x+(i+len), y)); len++);
    action |= countpentad(row, len, dest);
  }
  if(col_e){
    for(i = -1; i > -WINLEN && player != getsquare(x, y+i); i--);
    i++;
    for(len = 0; i+len < WINLEN && player != (row[len] = getsquare(x, y+(i+len))); len++);
    action |= countpentad(row, len, dest);
  }
  if(diag_e){
    for(i = -1; i > -WINLEN && player != getsquare(x+i, y+i); i--);
    i++;
    for(len = 0; i+len < WINLEN && player != (row[len] = getsquare(x+(i+len), y+(i+len))); len++);
    action |= countpentad(row, len, dest);

    for(i = -1; i > -WINLEN && player != getsquare(x-i, y+i); i--);
    i++;
    for(len = 0; i+len < WINLEN && player != (row[len] = getsquare(x-(i+len), y+(i+len))); len++);
    action |= countpentad(row, len, dest);
  }

  return action;
}

void val_get (int index, char player, mvalue *dest)
{
  char attack[WINLEN], defense[WINLEN];
  int i, x, y;

  index2coor(index, &x, &y);

  dest->attack = countsquare(x, y, player, attack);
  dest->defense = (countsquare(x, y, -player, defense) == 3);

  for(i=0; i<WINLEN-1; i++) dest->a[2*i] = attack[i]-attack[i+1];
  dest->a[2*i] = attack[i];
  for(i=1; i<WINLEN; i++) dest->a[2*i-1] = defense[i];
}

void val_invert (mvalue *v)
{
  int i;

  for(i=0; i<MVAL_LEN; i++) v->a[i] *= -1;
}

void val_swapplayers (mvalue *v)
{
  int i, tmp;

  for(i=0; i<MVAL_LEN-1; i+=2){
    tmp = v->a[i];
    v->a[i] = v->a[i+1];
    v->a[i+1] = tmp;
  }
}

void val_plusplus (mvalue *v)
{
  v->a[0]++;
}

void val_add (mvalue *v, mvalue *a)
{
  int i;

  for(i=0; i<MVAL_LEN; i++)
    v->a[i] += a->a[i];
}

char val_isinf (mvalue *v)
{
  return v->a[MVAL_LEN-1] > 0;
}

void val_print (mvalue *v)
{
  int i;

  printf("(");
  for(i=0; i<MVAL_LEN; i++){
    if(i) printf(", ");
    printf("%d", v->a[i]);
  }
  printf(")\n");
}

int val_cmp (const mvalue *v1, const mvalue *v2)
{
  int i;

  for(i = MVAL_LEN-1; i>=0; i--){
    if(v1->a[i] > v2->a[i]) return 1;
    if(v1->a[i] < v2->a[i]) return -1;
  }

  return 0;
}

static int gcd (int a, int b)
{
  int c;

  while(b){
    c = a%b;
    a = b;
    b = c;
  }
  return a;
}

void val_init ()
{
  int i;

  row_e = col_e = diag_e = 0;
  if(width >= WINLEN) row_e = 1;
  if(height >= WINLEN) col_e = 1;
  if(width/gcd(width, height)*height >= WINLEN) diag_e = 1;

  srand(time(NULL));

  for(i=0; i<MVAL_LEN; i++){
    val_pinf.a[i] = INT_MAX/2;
    val_minf.a[i] = INT_MIN/2;
    val_neut.a[i] = 0;
  }
}

void val_randomize (mvalue *v)
{
  v->a[0] += rand()%10;
  v->a[1] += rand()%6;
  v->a[2] += rand()%4;
  v->a[3] += rand()%2;
}

mvalue val_create (int a0, int a1, int a2, int a3, int a4,
	   int a5, int a6, int a7, int a8)
{
  mvalue result;

  result.a[0] = a0;
  result.a[1] = a1;
  result.a[2] = a2;
  result.a[3] = a3;
  result.a[4] = a4;
  result.a[5] = a5;
  result.a[6] = a6;
  result.a[7] = a7;
  result.a[8] = a8;

  return result;
}
