#include <gtk/gtk.h>
#include <sys/types.h>
#include <signal.h>
#include <stdlib.h>
#include "field.h"
#include "table.h"
#include "statusbar.h"
#include "spawn.h"

static GPid pid, killed = 0;

static char running = 0;

static void child_watch(GPid pid, gint status)
{
  g_spawn_close_pid(pid);
  running--;

  if(running < 0){
    fprintf(stderr, "spawn.c: running < 0\n");
    running = 0;
  }
}

static void out_watch(GIOChannel *channel, GIOCondition cond, gpointer p)
{
  gchar *str;
  int index;
  GPid pi = (int)(long int)p;

  if(pi == killed){
    g_io_channel_close(channel);
    return;
  }

  g_io_channel_read_line(channel, &str, NULL, NULL, NULL);
  if(!str) ai_click(-1);
  else{
    index = atoi(str);
    ai_click(index);
    g_free(str);
  }

  g_io_channel_close(channel);
}

void ai_play()
{
  char *argv[3];
  gint in, out;
  GIOChannel *in_ch, *out_ch;
  char str[20], ch;
  int i;

  argv[0] = "./piskvorky_ai";
  if(cur_player == -1){
    argv[1] = "-O";
    argv[2] = NULL;
  }
  else argv[1] = NULL;

  if(!g_spawn_async_with_pipes(NULL, argv, NULL,
			       G_SPAWN_DO_NOT_REAP_CHILD, NULL,
			       NULL, &pid, &in, &out, NULL, NULL)){
    ai_click(-1);
    return;
  }

  running++;
  g_child_watch_add(pid, (GChildWatchFunc)child_watch, NULL);

  in_ch = g_io_channel_unix_new(in);
  sprintf(str, "%d %d\n", height, width);
  g_io_channel_write_chars(in_ch, str, -1, NULL, NULL);
  for(i=0; i<size; i++){
    switch(field[i]){
    case -1: ch = 'O'; break;
    case 1: ch = 'X'; break;
    case 0: ch = '.'; break;
    default: ch = ' ';
    }
    g_io_channel_write_chars(in_ch, &ch, 1, NULL, NULL);
  }
  g_io_channel_unref(in_ch);

  out_ch = g_io_channel_unix_new(out);
  g_io_add_watch(out_ch, G_IO_HUP, (GIOFunc)out_watch, (gpointer)(long int)pid);
}

void ai_kill()
{
  if(!running || pid == killed) return;
  kill(pid, SIGKILL);
  killed = pid;
}
