#include <stdlib.h>
#include <gtk/gtk.h>
#include "base.h"
#include "piskvorky.h"
#include "field.h"
#include "statusbar.h"
#include "move.h"
#include "new.h"

static GdkColor blue, green, hblue, hgreen;
static GtkWidget *table, *tablebox;
static struct gsquare{
  GtkWidget *button;
  int index;
} *gfield;
static int highlighted[WINLEN+1];

static char ishlight(int index)
{
  int i;

  for(i=0; highlighted[i] >= 0; i++)
    if(highlighted[i] == index) return 1;
  return 0;
}

static void setbutcol(GtkWidget *button, GdkColor *col)
{
  gtk_widget_modify_bg (button, GTK_STATE_NORMAL, col);
  gtk_widget_modify_bg (button, GTK_STATE_ACTIVE, col);
  gtk_widget_modify_bg (button, GTK_STATE_PRELIGHT, col);
}

static int shift;

static struct gsquare *getgsquare(int index)
{
  index += shift;
  if(index%width < shift%width) index -= width;
  if(index >= size) index -= size;

  return &gfield[index];
}

static void fixsquare(struct gsquare *s)
{
  if(field[s->index] > 0){
    gtk_button_set_label(GTK_BUTTON(s->button), "X");
    if(ishlight(s->index)) setbutcol(s->button, &hgreen);
    else setbutcol(s->button, &green);
  }
  else if(field[s->index] < 0){
    gtk_button_set_label(GTK_BUTTON(s->button), "O");
    if(ishlight(s->index)) setbutcol(s->button, &hblue);
    else setbutcol(s->button, &blue);
  }
  else{
    gtk_button_set_label(GTK_BUTTON(s->button), "");
    setbutcol(s->button, NULL);
  }
}

static void unhlight()
{
  struct gsquare *s;
  int i;

  if(highlighted[0] < 0) return;

  s = getgsquare(highlighted[0]);
  highlighted[0] = -1;
  fixsquare(s);

  for(i=1; highlighted[i] >= 0; i++)
    fixsquare(getgsquare(highlighted[i]));
}

static void highlight()
{
  int i;

  for(i=0; highlighted[i] >= 0; i++)
    fixsquare(getgsquare(highlighted[i]));
}

void ai_click(int index)
{
  unhlight();

  if(index < 0 || index >= size || field[index]){
    set_finished(cur_player*2);
    return;
  }

  move(index, highlighted);
  if(highlighted[0] < 0) fixsquare(getgsquare(index));
  else highlight();
}

static void square_click(GtkWidget *button, gpointer data)
{
  struct gsquare *s;

  if(ai_playing || finished) return;

  s = ((struct gsquare *)data);

  if(field[s->index]) return;
  field[s->index] = cur_player;

  unhlight();
  move(s->index, highlighted);
  if(highlighted[0] < 0) fixsquare(s);
  else highlight();
}

static void shift_torus(GtkWidget *widget, gpointer data)
{
  int i;
  struct gsquare *s;

  switch((long int)data){
  case 1:
    shift -= width;
    if(shift < height) shift += size;
    break;
  case 0:
    shift += width;
    if(shift >= size) shift -= size;
    break;
  case 3:
    if(shift % width == 0) shift += width;
    shift--;
    break;
  case 2:
    shift++;
    if(shift % width == 0) shift -= width;
    break;
  }

  for(i=0; i<size; i++){
    s = getgsquare(i);
    s->index = i;
    fixsquare(s);
  }
}

void create_table(GtkWidget *box)
{
  GtkWidget *button;
  int i, j, index;

  tablebox = box;
  highlighted[0] = -1;

  shift = 0;

  gdk_color_parse ("#66ff66", &green);
  gdk_color_parse ("#6666ff", &blue);
  gdk_color_parse ("#aaffaa", &hgreen);
  gdk_color_parse ("#aaaaff", &hblue);

  table = gtk_table_new (height, width, TRUE);
  gtk_container_add (GTK_CONTAINER (box), table);

  gfield = (struct gsquare *)malloc(sizeof(struct gsquare)*size);

  index = 0;
  for(j=0; j<height; j++)
    for(i=0; i<width; i++){
      gfield[index].index = index;
      button = gfield[index].button = gtk_button_new_with_label(NULL);
      gtk_signal_connect (GTK_OBJECT (button), "clicked",
			  GTK_SIGNAL_FUNC (square_click), (gpointer)&gfield[index]);
      gtk_table_attach_defaults (GTK_TABLE(table), button, i+1, i+2, j+1, j+2);
      gtk_widget_show (button);
      index++;
    }

  button = gtk_button_new_with_label ("New");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (ngame_dialog), (gpointer) NULL);
  gtk_table_attach_defaults (GTK_TABLE(table), button, 0, 1, 0, 1);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("^");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (shift_torus), (gpointer) 0);
  gtk_table_attach_defaults (GTK_TABLE(table), button, 1, width+1, 0, 1);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("v");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (shift_torus), (gpointer) 1);
  gtk_table_attach_defaults (GTK_TABLE(table), button, 1, width+1, height+1, height+2);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("<");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (shift_torus), (gpointer) 2);
  gtk_table_attach_defaults (GTK_TABLE(table), button, 0, 1, 1, height+1);
  gtk_widget_show (button);

  button = gtk_button_new_with_label (">");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (shift_torus), (gpointer) 3);
  gtk_table_attach_defaults (GTK_TABLE(table), button, width+1, width+2, 1, height+1);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("Quit");
  gtk_signal_connect (GTK_OBJECT(button), "clicked",
		      GTK_SIGNAL_FUNC (quit), NULL);
  gtk_table_attach_defaults (GTK_TABLE(table), button, width+1, width+2, height+1, height+2);

  gtk_widget_show(button);

  gtk_widget_show(table);
}

void table_reset(int w, int h)
{
  int i;

  unhlight();

  if(w == width && h == height){
    for(i=0; i<size; i++){
      if(!field[i]) continue;
      field[i] = 0;
      fixsquare(getgsquare(i));
    }
    freesquares = size;
    return;
  }

  free(field);
  free(gfield);
  initfield(w, h);
  gtk_container_remove(GTK_CONTAINER(tablebox), table);
  create_table(tablebox);
}
